package net.scheinerman.grapher;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.scheinerman.toolbox.gui.JCenteredFrame;

public class GraphFrame extends JCenteredFrame {

	private static final long serialVersionUID = 1L;

	private ArrayList<GraphFunction> functions = new ArrayList<GraphFunction>();
	
	private JPanel mainPanel = new JPanel(new BorderLayout());
		private GraphPanel graph = new GraphPanel(functions);
		private JPanel functionEditPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
			private JLabel yEqualsLabel = new JLabel("y=");
			private JTextField functionEditField = new JTextField(28);
			private JButton graphButton = new JButton("Graph");
	private JPanel functionsPanel = new JPanel(new BorderLayout());
		private JPanel functionsTop = new JPanel(new FlowLayout(FlowLayout.CENTER));
			private JLabel functionsLabel = new JLabel("Functions:");
		private DefaultListModel functionsListModel = new DefaultListModel();
		private JList functionsList = new JList(functionsListModel);
		private JScrollPane functionsScroll = new JScrollPane(functionsList);
		private JPanel alterFunctionsPanel = new JPanel(new GridLayout(1, 2));
			private JButton addFunction = new JButton("+");
			private JButton rmvFunction = new JButton("-");
	
	public GraphFrame() {
		super("Grapher", 800, 500);
		constructGUI();
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setVisible(true);
	}
	
	private void constructGUI() {
		add(BorderLayout.CENTER, mainPanel);
			mainPanel.add(BorderLayout.CENTER, graph);
			mainPanel.add(BorderLayout.SOUTH, functionEditPanel);
				functionEditPanel.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
				functionEditPanel.add(yEqualsLabel);	
					yEqualsLabel.setEnabled(false);
				functionEditPanel.add(functionEditField);	
					functionEditField.setEnabled(false);
					functionEditField.addActionListener(new UpdateGraphAction());
				functionEditPanel.add(graphButton);
					graphButton.addActionListener(new UpdateGraphAction());
		add(BorderLayout.WEST, functionsPanel);
			functionsPanel.add(BorderLayout.NORTH, functionsTop);
				functionsTop.add(functionsLabel);
			functionsPanel.add(BorderLayout.CENTER, functionsScroll);
				functionsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
				functionsList.addListSelectionListener(new SelectFunctionAction());
			functionsPanel.add(BorderLayout.SOUTH, alterFunctionsPanel);
				alterFunctionsPanel.add(addFunction);
					addFunction.addActionListener(new AddFunctionAction());
				alterFunctionsPanel.add(rmvFunction);
					rmvFunction.addActionListener(new RmvFunctionAction());
	}
	
	private void addFunction() {
		GraphFunction function = new GraphFunction();
		functions.add(function);
		functionsListModel.addElement(function);
		functionsList.setSelectedValue(function, true);
	}
	
	private void rmvFunction() {
		int index = functionsList.getSelectedIndex();
		System.out.println(index);
		if(index != -1) {
			functions.remove(index);
			functionsListModel.remove(functionsList.getSelectedIndex());
			graph.repaint();
		}
	}
	
	private void selectFunction() {
		if(functionsList.getSelectedIndex() != -1) {
			GraphFunction function = (GraphFunction)functionsList.getSelectedValue();
			function.setSelected(true);
			yEqualsLabel.setEnabled(true);
			functionEditField.setEnabled(true);
			functionEditField.setText(function.getFunctionCode());
		} else {
			yEqualsLabel.setEnabled(false);
			functionEditField.setEnabled(false);
		}
	}
	
	private class UpdateGraphAction implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			if(functionEditField.isEnabled()) {
				GraphFunction function = (GraphFunction)functionsList.getSelectedValue();
				function.setFunctionCode(functionEditField.getText());
				functionsList.repaint();
			}
			graph.repaint();
		}		
	}
	
	private class AddFunctionAction implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			addFunction();
		}	
	}
	
	private class RmvFunctionAction implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			rmvFunction();
		}	
	}
	
	private class SelectFunctionAction implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent e) {
			selectFunction();
		}
	}
}
