package net.scheinerman.grapher;

import net.scheinerman.phoenix.interpreter.functions.BuiltInFunction;
import net.scheinerman.phoenix.interpreter.variables.NumberVariable;
import net.scheinerman.phoenix.library.AbstractModule;

public class GrapherPhoenixModule extends AbstractModule {

	public GrapherPhoenixModule() {
		super("");
		functions.add(new Abs());
		functions.add(new ArcCos());
		functions.add(new ArcSin());
		functions.add(new ArcTan());
		functions.add(new Ceil());
		functions.add(new Cos());
		functions.add(new CosH());
		functions.add(new E());
		functions.add(new Exp());
		functions.add(new Factorial());
		functions.add(new Floor());
		functions.add(new Hypot());
		functions.add(new Log());
		functions.add(new Log10());
		functions.add(new LogB());
		functions.add(new Max());
		functions.add(new Min());
		functions.add(new PI());
		functions.add(new Root());
		functions.add(new Sin());
		functions.add(new SinH());
		functions.add(new Sqrt());
		functions.add(new Tan());
		functions.add(new TanH());
	}
	
	private class Abs extends BuiltInFunction {
		public Abs() {
			super("abs", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.abs(d));
		}
	}
	
	private class ArcCos extends BuiltInFunction {
		public ArcCos() {
			super( "arccos", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.acos(d));
		}
	}

	
	private class ArcSin extends BuiltInFunction {
		public ArcSin() {
			super( "arcsin", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.asin(d));
		}
	}
	
	
	private class ArcTan extends BuiltInFunction {
		public ArcTan() {
			super( "arctan", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.atan(d));
		}
	}
	
	private class Ceil extends BuiltInFunction {
		public Ceil() {
			super( "ceil", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"n"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.ceil(d));
		}
	}
	
	private class Cos extends BuiltInFunction {
		public Cos() {
			super( "cos", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.cos(d));
		}
	}	
	
	private class CosH extends BuiltInFunction {
		public CosH() {
			super( "cosh", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.cosh(d));
		}
	}
	
	private class E extends BuiltInFunction {
		public E() {
			super( "E", "num", new String[] {}, new String[] {},
					new String[] {}, new String[] {});
		}
		public void run(String[] left, String[] right) {
			retValue = new NumberVariable(Math.E);
		}
	}
	
	private class Exp extends BuiltInFunction {
		public Exp() {
			super( "exp", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.exp(d));
		}
	}	
	
	private class Factorial extends BuiltInFunction {
		public Factorial() {
			super( "factorial", "num", new String[] {"num"}, new String[] {},
					new String[] {"n"}, new String[] {});
		}
		public void run(String[] left, String[] right) {
			try {
				double i = Double.parseDouble(left[0]);
				if(i < 0) {
					throw new IllegalArgumentException("Argument to factorial must be a non negative integer.");
				}
				double prod = 1;
				for(int loop = 2; loop <= i; loop++) { 
					prod *= loop; 
				}
				retValue = new NumberVariable(prod);
			} catch(NumberFormatException e) {
				throw new IllegalArgumentException("Argument to factorial must be a non negative integer.");
			}
		}
	}
	
	private class Floor extends BuiltInFunction {
		public Floor() {
			super( "floor", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"n"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.floor(d));
		}
	}
	
	private class Hypot extends BuiltInFunction {
		public Hypot() {
			super( "hypot", "num", new String[] {}, new String[] {"num", "num"},
					new String[] {}, new String[] {"x", "y"});
		}
		public void run(String[] left, String[] right) {
			Double d1 = Double.parseDouble(right[0]);
			Double d2 = Double.parseDouble(right[1]);
			retValue = new NumberVariable(Math.hypot(d1, d2));
		}
	}
	
	private class Log extends BuiltInFunction {
		public Log() {
			super( "log", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.log(d));
		}
	}
	
	private class Log10 extends BuiltInFunction {
		public Log10() {
			super( "log10", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.log10(d));
		}
	}
	
	private class LogB extends BuiltInFunction {
		public LogB() {
			super( "logb", "num", new String[] {}, new String[] {"num", "num"},
					new String[] {}, new String[] {"base", "x"});
		}
		public void run(String[] left, String[] right) {
			Double b = Double.parseDouble(right[0]);
			Double n = Double.parseDouble(right[1]);
			retValue = new NumberVariable(Math.log(n) / Math.log(b));
		}
	}
	
	private class Max extends BuiltInFunction {
		public Max() {
			super( "max", "num", new String[] {}, new String[] {"num", "num"},
					new String[] {}, new String[] {"a", "b"});
		}
		public void run(String[] left, String[] right) {
			Double a = Double.parseDouble(right[0]);
			Double b = Double.parseDouble(right[1]);
			retValue = new NumberVariable(Math.max(a,b));
		}
	}
		
	private class Min extends BuiltInFunction {
		public Min() {
			super( "min", "num", new String[] {}, new String[] {"num", "num"},
					new String[] {}, new String[] {"a", "b"});
		}
		public void run(String[] left, String[] right) {
			Double a = Double.parseDouble(right[0]);
			Double b = Double.parseDouble(right[1]);
			retValue = new NumberVariable(Math.min(a,b));
		}
	}	
	
	private class PI extends BuiltInFunction {
		public PI() {
			super( "PI", "num", new String[] {}, new String[] {},
					new String[] {}, new String[] {});
		}
		public void run(String[] left, String[] right) {
			retValue = new NumberVariable(Math.PI);
		}
	}
	
	private class Root extends BuiltInFunction {
		public Root() {
			super( "root", "num", new String[] {"num"}, new String[] {"num"},
					new String[] {"n"}, new String[] {"x"});
			description = "Calculates the nth root of the number x.";
			argDescriptions.put("n", "The index (the type of root).");
			argDescriptions.put("x", "The radicand (the value under the radical).");
			returnDescription = "The nth root of x.";
		}
		public void run(String[] left, String[] right) {
			Double a = Double.parseDouble(left[0]);
			Double b = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.pow(b, (1./a)));
		}
	}	
		
	private class Sin extends BuiltInFunction {
		public Sin() {
			super( "sin", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.sin(d));
		}
	}	
	
	private class SinH extends BuiltInFunction {
		public SinH() {
			super( "sinh", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.sinh(d));
		}
	}
	
	private class Sqrt extends BuiltInFunction {
		public Sqrt() {
			super( "sqrt", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.sqrt(d));
		}
	}
	private class Tan extends BuiltInFunction {
		public Tan() {
			super( "tan", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.tan(d));
		}
	}	
	
	private class TanH extends BuiltInFunction {
		public TanH() {
			super( "tanh", "num", new String[] {}, new String[] {"num"},
					new String[] {}, new String[] {"x"});
		}
		public void run(String[] left, String[] right) {
			Double d = Double.parseDouble(right[0]);
			retValue = new NumberVariable(Math.tanh(d));
		}
	}
	


}
